<?php
/**
 * Copyright © 2015 FiloBlu . All rights reserved.
 */
namespace FiloBlu\Flow\Helper;

class Images extends \Magento\Framework\App\Helper\AbstractHelper
{
    protected $_objectManager;
    protected $connection;

    protected $_configurable_type;
    protected $_product_factory;
    protected $myconnection;
    protected $resource;

    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        \Magento\Framework\App\ResourceConnection $connection,
        \Magento\Catalog\Model\ProductFactory $productFactory,
        \Magento\Framework\App\ResourceConnection $resource,
        \Magento\ConfigurableProduct\Model\ResourceModel\Product\Type\Configurable $configurableProduct
    ) {
        $this->connection=$connection;
        $this->_configurable_type = $configurableProduct;
        $this->_product_factory = $productFactory;
        $this->resource = $resource;
        parent::__construct($context);
    }

    public function isFullPhotoCleanActive(){
        $flag = $this->scopeConfig->getValue('filoblu_flow/flow_images/flow_images_force_clean', \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE);
        return $flag;
    }

    public function isClearImageCacheActive(){
        $flag = $this->scopeConfig->getValue('filoblu_flow/flow_images/flow_images_clean_cache', \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE);
        return $flag;
    }

    public function isUniqueUploadImagesActive(){
        $flag = $this->scopeConfig->getValue('filoblu_flow/flow_images/flow_images_upload_unique', \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE);
        return $flag;
    }

    public function isDuplicateImagesActive(){
        $flag = $this->scopeConfig->getValue('filoblu_flow/flow_images/flow_images_create_duplicates', \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE);
        return $flag;
    }

    /*
     * Returns an array of folders of $source_dir
     */
    public function foldersMap($source_dir, $directory_depth = 0, $hidden = FALSE)
    {
        if ($fp = @opendir($source_dir))
        {
            $filedata   = array();
            $new_depth  = $directory_depth - 1;
            $source_dir = rtrim($source_dir, '/').'/';

            while (FALSE !== ($file = readdir($fp)))
            {
                // Remove '.', '..', and hidden files [optional]
                if ( ! trim($file, '.') OR ($hidden == FALSE && $file[0] == '.'))
                {
                    continue;
                }

                if (($directory_depth < 1 OR $new_depth > 0) && @is_dir($source_dir.$file))
                {
                    $filedata[$file] = $this->foldersMap($source_dir.$file.'/', $new_depth, $hidden);
                }
                else
                {
                    $filedata[] = $file;
                }
            }

            closedir($fp);
            return $filedata;
        }
        echo 'can not open dir';
        return FALSE;
    }

    /*
     * This function will get as input one array of folders from foldersMap function
     * Then will convert that array into paths (strings) of the magento
     * images cache folder checking that the last folder is an MD5 (because this is how Magento builds the image caches paths)
     * https://www.sonassi.com/blog/knowledge-base/deconstructing-the-cache-image-path-on-magento
     *
     * $result = array();
     * collapse("",$array_of_paths,$result);
     *
     * $result will be an array of paths (strings)
     */
    public function collapse($path, $collapse, &$result)
    {
        foreach($collapse AS $key => $value)
        {
            if(is_array($value))
            {
                $this->collapse($path . $key . DIRECTORY_SEPARATOR, $value, $result);
                continue;
            }

            if(preg_match('/^[a-f0-9]{32}$/', $value)){
                $result[] = $path . $value . DIRECTORY_SEPARATOR;
            } else {
                $result[] = $path;
            }

        }
    }

    /*
     * This function will return all the products childs of a parent that have the same super_attibute value
     */
    public function getAllChildsOfParentWithSameAttribute($child_sku,$super_attribute_code){

        $child_skus = array();

        $child_id = $this->fastGetIdBySku($child_sku);

        if(!is_numeric($child_id)){
            return $child_skus;
        }

        $loaded_child = $this->_product_factory->create()->load($child_id);

        //$super_attribute_value = $loaded_child->getResource()->getAttribute($super_attribute_code)->getFrontend()->getValue($loaded_child);
        $super_attribute_value = $loaded_child->getData($super_attribute_code);

        // Get the product parent
        if ($loaded_child->getTypeId() != \Magento\Catalog\Model\Product\Type::DEFAULT_TYPE) {
            return $child_skus;
        }

        $parentIds = $this->_configurable_type->getParentIdsByChild($child_id);

        if (count($parentIds) == 0) {
            return $child_skus;
        }

        foreach ($parentIds as $parentId) {
            $parent_product = $this->_product_factory->create()->load($parentId);
            // Get now all the childs of the parent
            $_children = $parent_product->getTypeInstance()->getUsedProducts($parent_product);
            foreach ($_children as $child){
                if($child->getData($super_attribute_code) == $super_attribute_value && $child->getSku() != $loaded_child->getSku()){
                    $child_skus[] = $child->getSku();
                }
            }
            // Handling only one parent_sku
            break;
        }

        return $child_skus;
    }

    protected function fastGetIdBySku($sku = null){

        if(!$sku){
            return false;
        }

        $connection = $this->getMyConnection();
        $sql = "SELECT entity_id FROM catalog_product_entity WHERE sku = '{$sku}'";
        $result = $connection->fetchRow($sql);

        if(isset($result['entity_id']) && $result['entity_id']){
            return $result['entity_id'];
        } else {
            return false;
        }
    }

    protected function getMyConnection()
    {
        if (!$this->myconnection) {
            $this->myconnection = $this->resource->getConnection('core_write');
        }
        return $this->myconnection;
    }

}


