<?php

namespace FiloBlu\Flow\Model\Parser;

/**
 * Class Csv
 * @package FiloBlu\Flow\Model\Parser
 */
class Csv extends AbstractParser
{

    public $map;
    public $pointer;
    public $header;
    public $check_mapping_count = 1;
    /**
     * @var \Monolog\Logger
     */
    protected $_logger;
    private $linesToSkip = 0;
    private $separator = ';';
    private $delimiter = '"';

    /**
     * Csv constructor.
     * @param \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider
     */
    public function __construct(
        \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider
    ) {
        $this->_logger = $loggerProvider->getLogger();
        parent::__construct($loggerProvider);
    }

    /**
     * @param $config
     * @return AbstractParser|void
     */
    public function setConfig($config)
    {
        $this->_config = $config;

        $this->separator = (String)@$this->_config['separator'];
        $this->delimiter = (String)@$this->_config['delimiter'];
        $this->linesToSkip = (String)@$this->_config['lines_to_skip'];
        $this->check_mapping_count = (String)@$this->_config['check_mapping_count'];

    }

    /**
     * @param $resource string the csv file
     * @return array|mixed
     * @throws \Exception
     */
    public function process($resource)
    {

        $returnArray = [];

        if (false === ($handle = @fopen($resource, 'r'))) {
            throw new \Exception("Unable to open {$resource}");
        }

        $count = 0;
        while (($data = fgetcsv($handle, 0, $this->separator)) !== false) {

            $count++;
            //Controlli

            if ($count == 1) {
                $this->header = $data;
            }

            if ($count <= $this->linesToSkip) {
                continue;
            }

            try {
                $returnArray[] = $this->parse($data);
            } catch (\Exception $e) {
                throw new \Exception($e->getMessage() . ' at row' . $count);
            }

        }

        fclose($handle);
        $channel = $this->getChannel();
        if (method_exists($channel, 'commitData') && !is_null($channel)) {
            $channel->commitData();
        }
        return $returnArray;
    }


}
